@@ -2,6 +2,8 @@
 
 CmdExtract::CmdExtract(CommandData *Cmd)
 {
+  CmdExtract::Cmd=Cmd;
+
   *ArcName=0;
 
   *DestFileName=0;
@@ -21,7 +23,7 @@ CmdExtract::~CmdExtract()
 }
 
 
-void CmdExtract::DoExtract(CommandData *Cmd)
+void CmdExtract::DoExtract()
 {
   PasswordCancelled=false;
   DataIO.SetCurrentCommand(Cmd->Command[0]);
@@ -39,7 +41,7 @@ void CmdExtract::DoExtract(CommandData *Cmd)
       SecPassword PrevCmdPassword;
       PrevCmdPassword=Cmd->Password;
 
-      EXTRACT_ARC_CODE Code=ExtractArchive(Cmd);
+      EXTRACT_ARC_CODE Code=ExtractArchive();
 
       // Restore Cmd->Password, which could be changed in IsArchive() call
       // for next header encrypted archive.
@@ -56,9 +58,7 @@ void CmdExtract::DoExtract(CommandData *Cmd)
       ErrHandler.GetErrorCode()!=RARX_BADPWD) // Not in case of wrong archive password.
   {
     if (!PasswordCancelled)
-    {
-      mprintf(St(MExtrNoFiles));
-    }
+      uiMsg(UIERROR_NOFILESTOEXTRACT,ArcName);
     ErrHandler.SetErrorCode(RARX_NOFILES);
   }
 #ifndef GUI
@@ -75,7 +75,7 @@ void CmdExtract::DoExtract(CommandData *Cmd)
 }
 
 
-void CmdExtract::ExtractArchiveInit(CommandData *Cmd,Archive &Arc)
+void CmdExtract::ExtractArchiveInit(Archive &Arc)
 {
   DataIO.UnpArcSize=Arc.FileLength();
 
@@ -100,14 +100,11 @@ void CmdExtract::ExtractArchiveInit(CommandData *Cmd,Archive &Arc)
 }
 
 
-EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
+EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
 {
   Archive Arc(Cmd);
   if (!Arc.WOpen(ArcName))
-  {
-    ErrHandler.SetErrorCode(RARX_OPEN);
     return EXTRACT_ARC_NEXT;
-  }
 
   if (!Arc.IsArchive(true))
   {
@@ -126,7 +123,7 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
   if (Arc.Volume && !Arc.FirstVolume)
   {
     wchar FirstVolName[NM];
-    VolNameToFirstName(ArcName,FirstVolName,Arc.NewNumbering);
+    VolNameToFirstName(ArcName,FirstVolName,ASIZE(FirstVolName),Arc.NewNumbering);
 
     // If several volume names from same volume set are specified
     // and current volume is not first in set and first volume is present
@@ -162,21 +159,20 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
     DataIO.TotalArcSize+=VolumeSetSize;
   }
 
-  ExtractArchiveInit(Cmd,Arc);
+  ExtractArchiveInit(Arc);
 
   if (*Cmd->Command=='T' || *Cmd->Command=='I')
     Cmd->Test=true;
 
 
-#ifndef GUI
   if (*Cmd->Command=='I')
+  {
+#ifndef GUI   
     Cmd->DisablePercentage=true;
-  else
-    if (Cmd->Test)
-      mprintf(St(MExtrTest),ArcName);
-    else
-      mprintf(St(MExtracting),ArcName);
 #endif
+  }
+  else
+    uiStartArchiveExtract(!Cmd->Test,ArcName);
 
   Arc.ViewComment();
 
@@ -187,7 +183,7 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
 
 
     bool Repeat=false;
-    if (!ExtractCurrentFile(Cmd,Arc,Size,Repeat))
+    if (!ExtractCurrentFile(Arc,Size,Repeat))
       if (Repeat)
       {
         // If we started extraction from not first volume and need to
@@ -210,7 +206,7 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
 }
 
 
-bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderSize,bool &Repeat)
+bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
 {
   wchar Command=Cmd->Command[0];
   if (HeaderSize==0)
@@ -287,7 +283,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
   {
     wchar CurVolName[NM];
     wcsncpyz(CurVolName,ArcName,ASIZE(CurVolName));
-    VolNameToFirstName(ArcName,ArcName,Arc.NewNumbering);
+    VolNameToFirstName(ArcName,ArcName,ASIZE(ArcName),Arc.NewNumbering);
 
     if (wcsicomp(ArcName,CurVolName)!=0 && FileExist(ArcName))
     {
@@ -300,7 +296,6 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
     if (!ReconstructDone)
     {
       ReconstructDone=true;
-
       if (RecVolumesRestore(Cmd,Arc.FileName,true))
       {
         Repeat=true;
@@ -345,7 +340,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
   {
     if (ExactMatch)
     {
-      Log(Arc.FileName,St(MUnpCannotMerge),ArcFileName);
+      uiMsg(UIERROR_NEEDPREVVOL,Arc.FileName,ArcFileName);
 #ifdef RARDLL
       Cmd->DllError=ERAR_BAD_DATA;
 #endif
@@ -359,8 +354,13 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
 
   if (ExactMatch || (SkipSolid=Arc.Solid)!=0)
   {
+    // First common call of uiStartFileExtract. It is done before overwrite
+    // prompts, so if SkipSolid state is changed below, we'll need to make
+    // additional uiStartFileExtract calls with updated parameters.
+    if (!uiStartFileExtract(ArcFileName,!Cmd->Test,Cmd->Test && Command!='I',SkipSolid))
+      return false;
 
-    ExtrPrepareName(Cmd,Arc,ArcFileName,DestFileName,ASIZE(DestFileName));
+    ExtrPrepareName(Arc,ArcFileName,DestFileName,ASIZE(DestFileName));
 
     // DestFileName can be set empty in case of excessive -ap switch.
     ExtrFile=!SkipSolid && *DestFileName!=0 && !Arc.FileHead.SplitBefore;
@@ -390,10 +390,10 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
     if (Arc.FileHead.Encrypted)
     {
 #ifdef RARDLL
-      if (!ExtrDllGetPassword(Cmd))
+      if (!ExtrDllGetPassword())
         return false;
 #else
-      if (!ExtrGetPassword(Cmd,Arc,ArcFileName))
+      if (!ExtrGetPassword(Arc,ArcFileName))
       {
         PasswordCancelled=true;
         return false;
@@ -424,12 +424,13 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
     if (!CheckUnpVer(Arc,ArcFileName))
     {
       ExtrFile=false;
-      ErrHandler.SetErrorCode(RARX_WARNING);
+      ErrHandler.SetErrorCode(RARX_FATAL);
 #ifdef RARDLL
       Cmd->DllError=ERAR_UNKNOWN_FORMAT;
 #endif
     }
 
+    
     File CurFile;
 
     bool LinkEntry=Arc.FileHead.RedirType!=FSREDIR_NONE;
@@ -440,7 +441,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
         // Overwrite prompt for symbolic and hard links.
         bool UserReject=false;
         if (FileExist(DestFileName) && !UserReject)
-          FileCreate(Cmd,NULL,DestFileName,ASIZE(DestFileName),Cmd->Overwrite,&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime);
+          FileCreate(Cmd,NULL,DestFileName,ASIZE(DestFileName),&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime);
         if (UserReject)
           ExtrFile=false;
       }
@@ -451,18 +452,25 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
         if (!ExtrFile || Command=='P' || Command=='I' || Command=='E' || Cmd->ExclPath==EXCL_SKIPWHOLEPATH)
           return true;
         TotalFileCount++;
-        ExtrCreateDir(Cmd,Arc,ArcFileName);
+        ExtrCreateDir(Arc,ArcFileName);
         return true;
       }
       else
         if (ExtrFile) // Create files and file copies (FSREDIR_FILECOPY).
-          ExtrFile=ExtrCreateFile(Cmd,Arc,CurFile);
+          ExtrFile=ExtrCreateFile(Arc,CurFile);
 
     if (!ExtrFile && Arc.Solid)
     {
       SkipSolid=true;
       ExtrFile=true;
 
+      // We changed SkipSolid, so we need to call uiStartFileExtract
+      // with "Skip" parameter to change the operation status 
+      // from "extracting" to "skipping". For example, it can be necessary
+      // if user answered "No" to overwrite prompt when unpacking
+      // a solid archive.
+      if (!uiStartFileExtract(ArcFileName,false,false,true))
+        return false;
     }
     if (ExtrFile)
     {
@@ -472,7 +480,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
       {
         if (!TestMode && Command!='P' && CurFile.IsDevice())
         {
-          Log(Arc.FileName,St(MInvalidName),DestFileName);
+          uiMsg(UIERROR_INVALIDNAME,Arc.FileName,DestFileName);
           ErrHandler.WriteError(Arc.FileName,DestFileName);
         }
         TotalFileCount++;
@@ -520,7 +528,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
           memcmp(Arc.FileHead.PswCheck,PswCheck,SIZE_PSWCHECK)!=0 &&
           !Arc.BrokenHeader)
       {
-        Log(Arc.FileName,St(MWrongPassword));
+        uiMsg(UIERROR_BADPSW,Arc.FileName);
         ErrHandler.SetErrorCode(RARX_BADPWD);
         WrongPassword=true;
       }
@@ -550,12 +558,12 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
         if (Type==FSREDIR_HARDLINK || Type==FSREDIR_FILECOPY)
         {
           wchar NameExisting[NM];
-          ExtrPrepareName(Cmd,Arc,Arc.FileHead.RedirName,NameExisting,ASIZE(NameExisting));
+          ExtrPrepareName(Arc,Arc.FileHead.RedirName,NameExisting,ASIZE(NameExisting));
           if (FileCreateMode && *NameExisting!=0) // *NameExisting can be 0 in case of excessive -ap switch.
             if (Type==FSREDIR_HARDLINK)
               LinkSuccess=ExtractHardlink(DestFileName,NameExisting,ASIZE(NameExisting));
             else
-              LinkSuccess=ExtractFileCopy(Cmd,CurFile,Arc.FileName,DestFileName,NameExisting,ASIZE(NameExisting));
+              LinkSuccess=ExtractFileCopy(CurFile,Arc.FileName,DestFileName,NameExisting,ASIZE(NameExisting));
         }
         else
           if (Type==FSREDIR_UNIXSYMLINK || Type==FSREDIR_WINSYMLINK || Type==FSREDIR_JUNCTION)
@@ -565,9 +573,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
           }
           else
           {
-#ifndef SFX_MODULE
-            Log(Arc.FileName,St(MUnknownExtra),DestFileName);
-#endif
+            uiMsg(UIERROR_UNKNOWNEXTRA, Arc.FileName, DestFileName);
             LinkSuccess=false;
           }
           
@@ -588,6 +594,13 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
             UnstoreFile(DataIO,Arc.FileHead.UnpSize);
           else
           {
+#ifdef _ANDROID
+            // malloc and new do not report memory allocation errors
+            // in Android, so if free memory is set, check it here
+            // trying to prevent crash.
+            if (Cmd->FreeMem!=0 && Cmd->FreeMem < Arc.FileHead.WinSize)
+              throw std::bad_alloc();
+#endif
             Unp->Init(Arc.FileHead.WinSize,Arc.FileHead.Solid);
             Unp->SetDestSize(Arc.FileHead.UnpSize);
 #ifndef SFX_MODULE
@@ -630,13 +643,9 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
           if (!WrongPassword)
             if (Arc.FileHead.Encrypted && (!Arc.FileHead.UsePswCheck || 
                 Arc.BrokenHeader) && !AnySolidDataUnpackedWell)
-            {
-              Log(Arc.FileName,St(MEncrBadCRC),ArcFileName);
-            }
+              uiMsg(UIERROR_CHECKSUMENC,Arc.FileName,ArcFileName);
             else
-            {
-              Log(Arc.FileName,St(MCRCFailed),ArcFileName);
-            }
+              uiMsg(UIERROR_CHECKSUM,Arc.FileName,ArcFileName);
           BrokenFile=true;
           ErrHandler.SetErrorCode(RARX_CRC);
 #ifdef RARDLL
@@ -684,8 +693,9 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
         CurFile.SetCloseFileTime(
           Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.FileHead.mtime,
           Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.FileHead.atime);
-        if (!Cmd->IgnoreGeneralAttr)
-          SetFileAttr(CurFile.FileName,Arc.FileHead.FileAttr);
+        if (!Cmd->IgnoreGeneralAttr && !SetFileAttr(CurFile.FileName,Arc.FileHead.FileAttr))
+          uiMsg(UIERROR_FILEATTR,Arc.FileName,CurFile.FileName);
+
         PrevExtracted=true;
       }
     }
@@ -706,6 +716,9 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderS
 
 void CmdExtract::UnstoreFile(ComprDataIO &DataIO,int64 DestUnpSize)
 {
+  // 512 KB and larger buffer reported to reduce performance on old XP
+  // computers with WDC WD2000JD HDD. According to test made by user
+  // 256 KB buffer is optimal.
   Array<byte> Buffer(0x40000);
   while (1)
   {
@@ -720,19 +733,15 @@ void CmdExtract::UnstoreFile(ComprDataIO &DataIO,int64 DestUnpSize)
 }
 
 
-bool CmdExtract::ExtractFileCopy(CommandData *Cmd,File &New,wchar *ArcName,wchar *NameNew,wchar *NameExisting,size_t NameExistingSize)
+bool CmdExtract::ExtractFileCopy(File &New,wchar *ArcName,wchar *NameNew,wchar *NameExisting,size_t NameExistingSize)
 {
-#ifdef _WIN_ALL
-  UnixSlashToDos(NameExisting,NameExisting,NameExistingSize);
-#elif defined(_UNIX)
-  DosSlashToUnix(NameExisting,NameExisting,NameExistingSize);
-#endif
+  SlashToNative(NameExisting,NameExisting,NameExistingSize);
+
   File Existing;
-  if (!Existing.Open(NameExisting))
+  if (!Existing.WOpen(NameExisting))
   {
-    ErrHandler.OpenErrorMsg(ArcName,NameExisting);
-    Log(ArcName,St(MCopyError),NameExisting,NameNew);
-    Log(ArcName,St(MCopyErrorHint));
+    uiMsg(UIERROR_FILECOPY,ArcName,NameExisting,NameNew);
+    uiMsg(UIERROR_FILECOPYHINT,ArcName);
 #ifdef RARDLL
     Cmd->DllError=ERAR_EREFERENCE;
 #endif
@@ -756,7 +765,7 @@ bool CmdExtract::ExtractFileCopy(CommandData *Cmd,File &New,wchar *ArcName,wchar
 }
 
 
-void CmdExtract::ExtrPrepareName(CommandData *Cmd,Archive &Arc,const wchar *ArcFileName,wchar *DestName,size_t DestSize)
+void CmdExtract::ExtrPrepareName(Archive &Arc,const wchar *ArcFileName,wchar *DestName,size_t DestSize)
 {
   wcsncpyz(DestName,Cmd->ExtrPath,DestSize);
 
@@ -772,7 +781,7 @@ void CmdExtract::ExtrPrepareName(CommandData *Cmd,Archive &Arc,const wchar *ArcF
   if (Cmd->AppendArcNameToPath)
   {
     wcsncatz(DestName,PointToName(Arc.FirstVolumeName),DestSize);
-    SetExt(DestName,NULL);
+    SetExt(DestName,NULL,DestSize);
     AddEndSlash(DestName,DestSize);
   }
 #endif
@@ -826,7 +835,7 @@ void CmdExtract::ExtrPrepareName(CommandData *Cmd,Archive &Arc,const wchar *ArcF
 
 
 #ifdef RARDLL
-bool CmdExtract::ExtrDllGetPassword(CommandData *Cmd)
+bool CmdExtract::ExtrDllGetPassword()
 {
   if (!Cmd->Password.IsSet())
   {
@@ -858,12 +867,14 @@ bool CmdExtract::ExtrDllGetPassword(CommandData *Cmd)
 
 
 #ifndef RARDLL
-bool CmdExtract::ExtrGetPassword(CommandData *Cmd,Archive &Arc,const wchar *ArcFileName)
+bool CmdExtract::ExtrGetPassword(Archive &Arc,const wchar *ArcFileName)
 {
   if (!Password.IsSet())
   {
-    if (!GetPassword(PASSWORD_FILE,ArcFileName,&Password))
+    if (!uiGetPassword(UIPASSWORD_FILE,ArcFileName,&Password))
     {
+      uiMsg(UIERROR_INCERRCOUNT);
+
       return false;
     }
   }
@@ -872,12 +883,12 @@ bool CmdExtract::ExtrGetPassword(CommandData *Cmd,Archive &Arc,const wchar *ArcF
     if (!PasswordAll && !Arc.FileHead.Solid)
     {
       eprintf(St(MUseCurPsw),ArcFileName);
-      switch(Cmd->AllYes ? 1:Ask(St(MYesNoAll)))
+      switch(Cmd->AllYes ? 1 : Ask(St(MYesNoAll)))
       {
         case -1:
           ErrHandler.Exit(RARX_USERBREAK);
         case 2:
-          if (!GetPassword(PASSWORD_FILE,ArcFileName,&Password))
+          if (!uiGetPassword(UIPASSWORD_FILE,ArcFileName,&Password))
             return false;
           break;
         case 3:
@@ -912,7 +923,7 @@ void CmdExtract::ConvertDosPassword(Archive &Arc,SecPassword &DestPwd)
 #endif
 
 
-void CmdExtract::ExtrCreateDir(CommandData *Cmd,Archive &Arc,const wchar *ArcFileName)
+void CmdExtract::ExtrCreateDir(Archive &Arc,const wchar *ArcFileName)
 {
   if (Cmd->Test)
   {
@@ -933,7 +944,7 @@ void CmdExtract::ExtrCreateDir(CommandData *Cmd,Archive &Arc,const wchar *ArcFil
       // File with name same as this directory exists. Propose user
       // to overwrite it.
       bool UserReject;
-      FileCreate(Cmd,NULL,DestFileName,ASIZE(DestFileName),Cmd->Overwrite,&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime);
+      FileCreate(Cmd,NULL,DestFileName,ASIZE(DestFileName),&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime);
       DirExist=false;
     }
     if (!DirExist)
@@ -959,8 +970,7 @@ void CmdExtract::ExtrCreateDir(CommandData *Cmd,Archive &Arc,const wchar *ArcFil
     }
     else
     {
-      Log(Arc.FileName,St(MExtrErrMkDir),DestFileName);
-      ErrHandler.CheckLongPathErrMsg(DestFileName);
+      uiMsg(UIERROR_DIRCREATE,Arc.FileName,DestFileName);
       ErrHandler.SysErrMsg();
 #ifdef RARDLL
       Cmd->DllError=ERAR_ECREATE;
@@ -982,7 +992,7 @@ void CmdExtract::ExtrCreateDir(CommandData *Cmd,Archive &Arc,const wchar *ArcFil
 }
 
 
-bool CmdExtract::ExtrCreateFile(CommandData *Cmd,Archive &Arc,File &CurFile)
+bool CmdExtract::ExtrCreateFile(Archive &Arc,File &CurFile)
 {
   bool Success=true;
   wchar Command=Cmd->Command[0];
@@ -995,29 +1005,29 @@ bool CmdExtract::ExtrCreateFile(CommandData *Cmd,Archive &Arc,File &CurFile)
     bool UserReject;
     // Specify "write only" mode to avoid OpenIndiana NAS problems
     // with SetFileTime and read+write files.
-    if (!FileCreate(Cmd,&CurFile,DestFileName,ASIZE(DestFileName),Cmd->Overwrite,&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,true))
+    if (!FileCreate(Cmd,&CurFile,DestFileName,ASIZE(DestFileName),&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,true))
     {
       Success=false;
       if (!UserReject)
       {
         ErrHandler.CreateErrorMsg(Arc.FileName,DestFileName);
-        ErrHandler.SetErrorCode(RARX_CREATE);
 #ifdef RARDLL
         Cmd->DllError=ERAR_ECREATE;
 #endif
         if (!IsNameUsable(DestFileName))
         {
-          Log(Arc.FileName,St(MCorrectingName));
+          uiMsg(UIMSG_CORRECTINGNAME,Arc.FileName);
+
           wchar OrigName[ASIZE(DestFileName)];
           wcsncpyz(OrigName,DestFileName,ASIZE(OrigName));
 
           MakeNameUsable(DestFileName,true);
 
           CreatePath(DestFileName,true);
-          if (FileCreate(Cmd,&CurFile,DestFileName,ASIZE(DestFileName),Cmd->Overwrite,&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,true))
+          if (FileCreate(Cmd,&CurFile,DestFileName,ASIZE(DestFileName),&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,true))
           {
 #ifndef SFX_MODULE
-            Log(Arc.FileName,St(MRenaming),OrigName,DestFileName);
+            uiMsg(UIERROR_RENAMING,Arc.FileName,OrigName,DestFileName);
 #endif
             Success=true;
           }
@@ -1051,13 +1061,8 @@ bool CmdExtract::CheckUnpVer(Archive &Arc,const wchar *ArcFileName)
 
   if (WrongVer)
   {
-#ifndef SILENT
-    Log(Arc.FileName,St(MUnknownMeth),ArcFileName);
-#ifndef SFX_MODULE
-//      Log(Arc.FileName,St(MVerRequired),Arc.FileHead.UnpVer/10,Arc.FileHead.UnpVer%10);
-    Log(Arc.FileName,St(MNewerRAR));
-#endif
-#endif
+    ErrHandler.UnknownMethodMsg(Arc.FileName,ArcFileName);
+    uiMsg(UIERROR_NEWERRAR,Arc.FileName);
   }
   return !WrongVer;
 }
