@@ -2,13 +2,15 @@ bool ExtractHardlink(wchar *NameNew,wchar *NameExisting,size_t NameExistingSize)
 {
   if (!FileExist(NameExisting))
     return false;
+  CreatePath(NameNew,true);
+
 #ifdef _WIN_ALL
   UnixSlashToDos(NameExisting,NameExisting,NameExistingSize);
 
   bool Success=CreateHardLink(NameNew,NameExisting,NULL)!=0;
   if (!Success)
   {
-    Log(NULL,St(MErrCreateLnkH),NameNew);
+    uiMsg(UIERROR_HLINKCREATE,NameNew);
     ErrHandler.SysErrMsg();
     ErrHandler.SetErrorCode(RARX_CREATE);
   }
@@ -22,7 +24,7 @@ bool ExtractHardlink(wchar *NameNew,wchar *NameExisting,size_t NameExistingSize)
   bool Success=link(NameExistingA,NameNewA)==0;
   if (!Success)
   {
-    Log(NULL,St(MErrCreateLnkH),NameNew);
+    uiMsg(UIERROR_HLINKCREATE,NameNew);
     ErrHandler.SysErrMsg();
     ErrHandler.SetErrorCode(RARX_CREATE);
   }
