@@ -71,44 +71,39 @@ void SetName(wchar *FullName,const wchar *Name,size_t MaxSize)
 }
 
 
-void SetExt(wchar *Name,const wchar *NewExt)
+void SetExt(wchar *Name,const wchar *NewExt,size_t MaxSize)
 {
   if (Name==NULL || *Name==0)
     return;
   wchar *Dot=GetExt(Name);
-  if (NewExt==NULL)
+  if (Dot!=NULL)
+    *Dot=0;
+  if (NewExt!=NULL)
   {
-    if (Dot!=NULL)
-      *Dot=0;
+    wcsncatz(Name,L".",MaxSize);
+    wcsncatz(Name,NewExt,MaxSize);
   }
-  else
-    if (Dot==NULL)
-    {
-      wcscat(Name,L".");
-      wcscat(Name,NewExt);
-    }
-    else
-      wcscpy(Dot+1,NewExt);
 }
 
 
 #ifndef SFX_MODULE
-void SetSFXExt(wchar *SFXName)
+void SetSFXExt(wchar *SFXName,size_t MaxSize)
 {
   if (SFXName==NULL || *SFXName==0)
     return;
 
 #ifdef _UNIX
-  SetExt(SFXName,L"sfx");
+  SetExt(SFXName,L"sfx",MaxSize);
 #endif
 
 #if defined(_WIN_ALL) || defined(_EMX)
-  SetExt(SFXName,L"exe");
+  SetExt(SFXName,L"exe",MaxSize);
 #endif
 }
 #endif
 
 
+// 'Ext' is an extension with the leading dot, like L".rar".
 wchar *GetExt(const wchar *Name)
 {
   return Name==NULL ? NULL:wcsrchr(PointToName(Name),'.');
@@ -162,7 +157,7 @@ void AddEndSlash(wchar *Path,size_t MaxLength)
 {
   size_t Length=wcslen(Path);
   if (Length>0 && Path[Length-1]!=CPATHDIVIDER && Length+1<MaxLength)
-    wcscat(Path,PATHDIVIDERW);
+    wcscat(Path,SPATHDIVIDER);
 }
 
 
@@ -424,6 +419,7 @@ void MakeNameUsable(char *Name,bool Extended)
 #ifndef _UNIX
     if (s-Name>1 && *s==':')
       *s='_';
+    // Remove ' ' and '.' before path separator, but allow .\ and ..\.
     if ((*s==' ' || *s=='.' && s>Name && !IsPathDiv(s[-1]) && s[-1]!='.') && IsPathDiv(s[1]))
       *s='_';
 #endif
@@ -440,6 +436,7 @@ void MakeNameUsable(wchar *Name,bool Extended)
 #ifndef _UNIX
     if (s-Name>1 && *s==':')
       *s='_';
+    // Remove ' ' and '.' before path separator, but allow .\ and ..\.
     if ((*s==' ' || *s=='.' && s>Name && !IsPathDiv(s[-1]) && s[-1]!='.') && IsPathDiv(s[1]))
       *s='_';
 #endif
@@ -447,91 +444,39 @@ void MakeNameUsable(wchar *Name,bool Extended)
 }
 
 
-char* UnixSlashToDos(char *SrcName,char *DestName,size_t MaxLength)
+void UnixSlashToDos(const char *SrcName,char *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (strlen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      strcpy(DestName,SrcName);
-  for (char *s=SrcName;*s!=0;s=charnext(s))
-  {
-    if (*s=='/')
-      if (DestName==NULL)
-        *s='\\';
-      else
-        DestName[s-SrcName]='\\';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='/' ? '\\':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
-char* DosSlashToUnix(char *SrcName,char *DestName,size_t MaxLength)
+void DosSlashToUnix(const char *SrcName,char *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (strlen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      strcpy(DestName,SrcName);
-  for (char *s=SrcName;*s!=0;s=charnext(s))
-  {
-    if (*s=='\\')
-      if (DestName==NULL)
-        *s='/';
-      else
-        DestName[s-SrcName]='/';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='\\' ? '/':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
-wchar* UnixSlashToDos(wchar *SrcName,wchar *DestName,size_t MaxLength)
+void UnixSlashToDos(const wchar *SrcName,wchar *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (wcslen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      wcscpy(DestName,SrcName);
-  for (wchar *s=SrcName;*s!=0;s++)
-  {
-    if (*s=='/')
-      if (DestName==NULL)
-        *s='\\';
-      else
-        DestName[s-SrcName]='\\';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='/' ? '\\':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
-wchar* DosSlashToUnix(wchar *SrcName,wchar *DestName,size_t MaxLength)
+void DosSlashToUnix(const wchar *SrcName,wchar *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (wcslen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      wcscpy(DestName,SrcName);
-  for (wchar *s=SrcName;*s!=0;s++)
-  {
-    if (*s=='\\')
-      if (DestName==NULL)
-        *s='/';
-      else
-        DestName[s-SrcName]='/';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='\\' ? '/':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
@@ -642,10 +587,10 @@ int ParseVersionFileName(wchar *Name,bool Truncate)
 
 #if !defined(SFX_MODULE) && !defined(SETUP)
 // Get the name of first volume. Return the leftmost digit of volume number.
-wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumbering)
+wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,size_t MaxSize,bool NewNumbering)
 {
   if (FirstName!=VolName)
-    wcscpy(FirstName,VolName);
+    wcsncpyz(FirstName,VolName,MaxSize);
   wchar *VolNumStart=FirstName;
   if (NewNumbering)
   {
@@ -668,7 +613,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
   else
   {
     // Old volume numbering scheme. Just set the extension to ".rar".
-    SetExt(FirstName,L"rar");
+    SetExt(FirstName,L"rar",MaxSize);
     VolNumStart=GetExt(FirstName);
   }
   if (!FileExist(FirstName))
@@ -677,8 +622,8 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
     // check if volume with same name and any other extension is available.
     // It can help in case of *.exe or *.sfx first volume.
     wchar Mask[NM];
-    wcscpy(Mask,FirstName);
-    SetExt(Mask,L"*");
+    wcsncpyz(Mask,FirstName,ASIZE(Mask));
+    SetExt(Mask,L"*",ASIZE(Mask));
     FindFile Find;
     Find.SetMask(Mask);
     FindData FD;
@@ -687,7 +632,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
       Archive Arc;
       if (Arc.Open(FD.Name,0) && Arc.IsArchive(true) && Arc.FirstVolume)
       {
-        wcscpy(FirstName,FD.Name);
+        wcsncpyz(FirstName,FD.Name,MaxSize);
         break;
       }
     }
