@@ -3,6 +3,7 @@
 #include "rarvmtbl.cpp"
 
 RarVM::RarVM()
+:BitInput(true)
 {
   Mem=NULL;
 }
@@ -20,56 +21,98 @@ void RarVM::Init()
     Mem=new byte[VM_MEMSIZE+4];
 }
 
+/*********************************************************************
+ IS_VM_MEM macro checks if address belongs to VM memory pool (Mem).
+ Only Mem data are always low endian regardless of machine architecture,
+ so we need to convert them to native format when reading or writing.
+ VM registers have endianness of host machine.
+**********************************************************************/
+#define IS_VM_MEM(a) (((byte*)a)>=Mem && ((byte*)a)<Mem+VM_MEMSIZE)
 
 inline uint RarVM::GetValue(bool ByteMode,uint *Addr)
 {
   if (ByteMode)
+  {
+#ifdef BIG_ENDIAN
+    if (IS_VM_MEM(Addr))
+      return(*(byte *)Addr);
+    else
+      return(*Addr & 0xff);
+#else
     return(*(byte *)Addr);
+#endif
+  }
   else
   {
-#if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT)
-    byte *B=(byte *)Addr;
-    return UINT32((uint)B[0]|((uint)B[1]<<8)|((uint)B[2]<<16)|((uint)B[3]<<24));
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED)
+    if (IS_VM_MEM(Addr))
+    {
+      byte *B=(byte *)Addr;
+      return GET_UINT32((uint)B[0]|((uint)B[1]<<8)|((uint)B[2]<<16)|((uint)B[3]<<24));
+    }
+    else
+      return GET_UINT32(*Addr);
 #else
-    return UINT32(*Addr);
+    return GET_UINT32(*Addr);
 #endif
   }
 }
 
-#if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT)
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED)
   #define GET_VALUE(ByteMode,Addr) GetValue(ByteMode,(uint *)Addr)
 #else
-  #define GET_VALUE(ByteMode,Addr) ((ByteMode) ? (*(byte *)(Addr)):UINT32(*(uint *)(Addr)))
+  #define GET_VALUE(ByteMode,Addr) ((ByteMode) ? (*(byte *)(Addr)):GET_UINT32(*(uint *)(Addr)))
 #endif
 
 
 inline void RarVM::SetValue(bool ByteMode,uint *Addr,uint Value)
 {
   if (ByteMode)
+  {
+#ifdef BIG_ENDIAN
+    if (IS_VM_MEM(Addr))
+      *(byte *)Addr=Value;
+    else
+      *Addr=(*Addr & ~0xff)|(Value & 0xff);
+#else
     *(byte *)Addr=Value;
+#endif
+  }
   else
   {
-#if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT) || !defined(PRESENT_INT32)
-    ((byte *)Addr)[0]=(byte)Value;
-    ((byte *)Addr)[1]=(byte)(Value>>8);
-    ((byte *)Addr)[2]=(byte)(Value>>16);
-    ((byte *)Addr)[3]=(byte)(Value>>24);
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED) || !defined(PRESENT_INT32)
+    if (IS_VM_MEM(Addr))
+    {
+      ((byte *)Addr)[0]=(byte)Value;
+      ((byte *)Addr)[1]=(byte)(Value>>8);
+      ((byte *)Addr)[2]=(byte)(Value>>16);
+      ((byte *)Addr)[3]=(byte)(Value>>24);
+    }
+    else
+      *(uint *)Addr=Value;
 #else
     *(uint32 *)Addr=Value;
 #endif
   }
 }
 
-#if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT) || !defined(PRESENT_INT32)
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED) || !defined(PRESENT_INT32)
   #define SET_VALUE(ByteMode,Addr,Value) SetValue(ByteMode,(uint *)Addr,Value)
 #else
-  #define SET_VALUE(ByteMode,Addr,Value) ((ByteMode) ? (*(byte *)(Addr)=(Value)):(*(uint32 *)(Addr)=((uint32)(Value))))
+  #define SET_VALUE(ByteMode,Addr,Value) ((ByteMode) ? (*(byte *)(Addr)=((byte)(Value))):(*(uint32 *)(Addr)=((uint32)(Value))))
 #endif
 
 
-void RarVM::SetValue(uint *Addr,uint Value)
+void RarVM::SetLowEndianValue(uint *Addr,uint Value)
 {
-  SetValue(false,Addr,Value);
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED) || !defined(PRESENT_INT32)
+  ((byte *)Addr)[0]=(byte)Value;
+  ((byte *)Addr)[1]=(byte)(Value>>8);
+  ((byte *)Addr)[2]=(byte)(Value>>16);
+  ((byte *)Addr)[3]=(byte)(Value>>24);
+#else
+  *(uint32 *)Addr=Value;
+#endif
 }
 
 
@@ -85,10 +128,10 @@ inline uint* RarVM::GetOperand(VM_PreparedOperand *CmdOp)
 void RarVM::Execute(VM_PreparedProgram *Prg)
 {
   memcpy(R,Prg->InitR,sizeof(Prg->InitR));
-  unsigned int GlobalSize=Min(Prg->GlobalData.Size(),VM_GLOBALMEMSIZE);
+  size_t GlobalSize=Min(Prg->GlobalData.Size(),VM_GLOBALMEMSIZE);
   if (GlobalSize)
     memcpy(Mem+VM_GLOBALMEMADDR,&Prg->GlobalData[0],GlobalSize);
-  unsigned int StaticSize=Min(Prg->StaticData.Size(),VM_GLOBALMEMSIZE-GlobalSize);
+  size_t StaticSize=Min(Prg->StaticData.Size(),VM_GLOBALMEMSIZE-GlobalSize);
   if (StaticSize)
     memcpy(Mem+VM_GLOBALMEMADDR+GlobalSize,&Prg->StaticData[0],StaticSize);
 
@@ -96,8 +139,11 @@ void RarVM::Execute(VM_PreparedProgram *Prg)
   Flags=0;
 
   VM_PreparedCommand *PreparedCode=Prg->AltCmd ? Prg->AltCmd:&Prg->Cmd[0];
-  if (!ExecuteCode(PreparedCode,Prg->CmdCount))
+  if (Prg->CmdCount>0 && !ExecuteCode(PreparedCode,Prg->CmdCount))
+  {
+    // Invalid VM program. Let's replace it with 'return' command.
     PreparedCode[0].OpCode=VM_RET;
+  }
   uint NewBlockPos=GET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20])&VM_MEMMASK;
   uint NewBlockSize=GET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x1c])&VM_MEMMASK;
   if (NewBlockPos+NewBlockSize>=VM_MEMSIZE)
@@ -106,7 +152,8 @@ void RarVM::Execute(VM_PreparedProgram *Prg)
   Prg->FilteredDataSize=NewBlockSize;
 
   Prg->GlobalData.Reset();
-  uint DataSize=Min(GET_VALUE(false,(uint*)&Mem[VM_GLOBALMEMADDR+0x30]),VM_GLOBALMEMSIZE);
+
+  uint DataSize=Min(GET_VALUE(false,(uint*)&Mem[VM_GLOBALMEMADDR+0x30]),VM_GLOBALMEMSIZE-VM_FIXEDGLOBALSIZE);
   if (DataSize!=0)
   {
     Prg->GlobalData.Add(DataSize+VM_FIXEDGLOBALSIZE);
@@ -115,6 +162,14 @@ void RarVM::Execute(VM_PreparedProgram *Prg)
 }
 
 
+/*
+Note:
+  Due to performance considerations RAR VM may set VM_FS, VM_FC, VM_FZ
+  incorrectly for byte operands. These flags are always valid only
+  for 32-bit operands. Check implementation of concrete VM command
+  to see if it sets flags right.
+*/
+
 #define SET_IP(IP)                      \
   if ((IP)>=CodeSize)                   \
     return(true);                       \
@@ -122,14 +177,17 @@ void RarVM::Execute(VM_PreparedProgram *Prg)
     return(false);                      \
   Cmd=PreparedCode+(IP);
 
-bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
+bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
 {
   int MaxOpCount=25000000;
   VM_PreparedCommand *Cmd=PreparedCode;
   while (1)
   {
+#ifndef NORARVM
+    // Get addresses to quickly access operands.
     uint *Op1=GetOperand(&Cmd->Op1);
     uint *Op2=GetOperand(&Cmd->Op2);
+#endif
     switch(Cmd->OpCode)
     {
 #ifndef NORARVM
@@ -147,7 +205,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
       case VM_CMP:
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
         }
         break;
@@ -155,14 +213,14 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
       case VM_CMPB:
         {
           uint Value1=GET_VALUE(true,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(true,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(true,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
         }
         break;
       case VM_CMPD:
         {
           uint Value1=GET_VALUE(false,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(false,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(false,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
         }
         break;
@@ -170,8 +228,14 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
       case VM_ADD:
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
-          uint Result=UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2));
-          Flags=Result==0 ? VM_FZ:(Result<Value1)|(Result&VM_FS);
+          uint Result=GET_UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2));
+          if (Cmd->ByteMode)
+          {
+            Result&=0xff;
+            Flags=(Result<Value1)|(Result==0 ? VM_FZ:((Result&0x80) ? VM_FS:0));
+          }
+          else
+            Flags=(Result<Value1)|(Result==0 ? VM_FZ:(Result&VM_FS));
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
@@ -186,7 +250,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
       case VM_SUB:
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -215,7 +279,9 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         break;
       case VM_INC:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)+1);
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)+1);
+          if (Cmd->ByteMode)
+            Result&=0xff;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
         }
@@ -230,7 +296,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
 #endif
       case VM_DEC:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)-1);
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)-1);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
         }
@@ -248,28 +314,28 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         continue;
       case VM_XOR:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)^GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)^GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_AND:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_OR:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)|GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)|GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_TEST:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
         }
         break;
@@ -335,7 +401,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint Value2=GET_VALUE(Cmd->ByteMode,Op2);
-          uint Result=UINT32(Value1<<Value2);
+          uint Result=GET_UINT32(Value1<<Value2);
           Flags=(Result==0 ? VM_FZ:(Result&VM_FS))|((Value1<<(Value2-1))&0x80000000 ? VM_FC:0);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -344,7 +410,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint Value2=GET_VALUE(Cmd->ByteMode,Op2);
-          uint Result=UINT32(Value1>>Value2);
+          uint Result=GET_UINT32(Value1>>Value2);
           Flags=(Result==0 ? VM_FZ:(Result&VM_FS))|((Value1>>(Value2-1))&VM_FC);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -353,24 +419,26 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint Value2=GET_VALUE(Cmd->ByteMode,Op2);
-          uint Result=UINT32(((int)Value1)>>Value2);
+          uint Result=GET_UINT32(((int)Value1)>>Value2);
           Flags=(Result==0 ? VM_FZ:(Result&VM_FS))|((Value1>>(Value2-1))&VM_FC);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_NEG:
         {
-          uint Result=UINT32(-GET_VALUE(Cmd->ByteMode,Op1));
+          // We use "0-value" expression to suppress "unary minus to unsigned"
+          // compiler warning.
+          uint Result=GET_UINT32(0-GET_VALUE(Cmd->ByteMode,Op1));
           Flags=Result==0 ? VM_FZ:VM_FC|(Result&VM_FS);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
 #ifdef VM_OPTIMIZE
       case VM_NEGB:
-        SET_VALUE(true,Op1,-GET_VALUE(true,Op1));
+        SET_VALUE(true,Op1,0-GET_VALUE(true,Op1));
         break;
       case VM_NEGD:
-        SET_VALUE(false,Op1,-GET_VALUE(false,Op1));
+        SET_VALUE(false,Op1,0-GET_VALUE(false,Op1));
         break;
 #endif
       case VM_PUSHA:
@@ -429,8 +497,10 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint FC=(Flags&VM_FC);
-          uint Result=UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2)+FC);
-          Flags=Result==0 ? VM_FZ:(Result<Value1 || Result==Value1 && FC)|(Result&VM_FS);
+          uint Result=GET_UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2)+FC);
+          if (Cmd->ByteMode)
+            Result&=0xff;
+          Flags=(Result<Value1 || Result==Value1 && FC)|(Result==0 ? VM_FZ:(Result&VM_FS));
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
@@ -438,12 +508,14 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint FC=(Flags&VM_FC);
-          uint Result=UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2)-FC);
-          Flags=Result==0 ? VM_FZ:(Result>Value1 || Result==Value1 && FC)|(Result&VM_FS);
+          uint Result=GET_UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2)-FC);
+          if (Cmd->ByteMode)
+            Result&=0xff;
+          Flags=(Result>Value1 || Result==Value1 && FC)|(Result==0 ? VM_FZ:(Result&VM_FS));
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
-#endif
+#endif  // for #ifndef NORARVM
       case VM_RET:
         if (R[7]>=VM_MEMSIZE)
           return(true);
@@ -453,7 +525,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
 #ifdef VM_STANDARDFILTERS
       case VM_STANDARD:
         ExecuteStandardFilter((VM_StandardFilters)Cmd->Op1.Data);
-        break;
+        return true;
 #endif
       case VM_PRINT:
         break;
@@ -466,24 +538,26 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,int CodeSize)
 
 
 
-void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
+void RarVM::Prepare(byte *Code,uint CodeSize,VM_PreparedProgram *Prg)
 {
   InitBitInput();
   memcpy(InBuf,Code,Min(CodeSize,BitInput::MAX_SIZE));
 
+  // Calculate the single byte XOR checksum to check validity of VM code.
   byte XorSum=0;
-  for (int I=1;I<CodeSize;I++)
+  for (uint I=1;I<CodeSize;I++)
     XorSum^=Code[I];
 
   faddbits(8);
 
   Prg->CmdCount=0;
-  if (XorSum==Code[0])
+  if (XorSum==Code[0]) // VM code is valid if equal.
   {
 #ifdef VM_STANDARDFILTERS
     VM_StandardFilters FilterType=IsStandardFilter(Code,CodeSize);
     if (FilterType!=VMSF_NONE)
     {
+      // VM code is found among standard filters.
       Prg->Cmd.Add(1);
       VM_PreparedCommand *CurCmd=&Prg->Cmd[Prg->CmdCount++];
       CurCmd->OpCode=VM_STANDARD;
@@ -492,21 +566,28 @@ void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
       CurCmd->Op2.Addr=&CurCmd->Op2.Data;
       CurCmd->Op1.Type=CurCmd->Op2.Type=VM_OPNONE;
       CodeSize=0;
+      return;
     }
 #endif  
+#ifndef NORARVM
     uint DataFlag=fgetbits();
     faddbits(1);
+
+    // Read static data contained in DB operators. This data cannot be
+    // changed, it is a part of VM code, not a filter parameter.
+
     if (DataFlag&0x8000)
     {
-      int DataSize=ReadData(*this)+1;
-      for (int I=0;InAddr<CodeSize && I<DataSize;I++)
+      uint DataSize=ReadData(*this)+1;
+      for (uint I=0;(uint)InAddr<CodeSize && I<DataSize;I++)
       {
         Prg->StaticData.Add(1);
         Prg->StaticData[I]=fgetbits()>>8;
         faddbits(8);
       }
     }
-    while (InAddr<CodeSize)
+
+    while ((uint)InAddr<CodeSize)
     {
       Prg->Cmd.Add(1);
       VM_PreparedCommand *CurCmd=&Prg->Cmd[Prg->CmdCount];
@@ -523,7 +604,7 @@ void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
       }
       if (VM_CmdFlags[CurCmd->OpCode] & VMCF_BYTEMODE)
       {
-        CurCmd->ByteMode=fgetbits()>>15;
+        CurCmd->ByteMode=(fgetbits()>>15)!=0;
         faddbits(1);
       }
       else
@@ -533,13 +614,14 @@ void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
       CurCmd->Op1.Addr=CurCmd->Op2.Addr=NULL;
       if (OpNum>0)
       {
-        DecodeArg(CurCmd->Op1,CurCmd->ByteMode);
+        DecodeArg(CurCmd->Op1,CurCmd->ByteMode); // reading the first operand
         if (OpNum==2)
-          DecodeArg(CurCmd->Op2,CurCmd->ByteMode);
+          DecodeArg(CurCmd->Op2,CurCmd->ByteMode); // reading the second operand
         else
         {
           if (CurCmd->Op1.Type==VM_OPINT && (VM_CmdFlags[CurCmd->OpCode]&(VMCF_JUMP|VMCF_PROC)))
           {
+            // Calculating jump distance.
             int Distance=CurCmd->Op1.Data;
             if (Distance>=256)
               Distance-=256;
@@ -561,7 +643,10 @@ void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
       }
       Prg->CmdCount++;
     }
+#endif
   }
+
+  // Adding RET command at the end of program.
   Prg->Cmd.Add(1);
   VM_PreparedCommand *CurCmd=&Prg->Cmd[Prg->CmdCount++];
   CurCmd->OpCode=VM_RET;
@@ -569,6 +654,11 @@ void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
   CurCmd->Op2.Addr=&CurCmd->Op2.Data;
   CurCmd->Op1.Type=CurCmd->Op2.Type=VM_OPNONE;
 
+  // If operand 'Addr' field has not been set by DecodeArg calls above,
+  // let's set it to point to operand 'Data' field. It is necessary for
+  // VM_OPINT type operands (usual integers) or maybe if something was 
+  // not set properly for other operands. 'Addr' field is required
+  // for quicker addressing of operand data.
   for (int I=0;I<Prg->CmdCount;I++)
   {
     VM_PreparedCommand *Cmd=&Prg->Cmd[I];
@@ -585,36 +675,39 @@ void RarVM::Prepare(byte *Code,int CodeSize,VM_PreparedProgram *Prg)
 }
 
 
+#ifndef NORARVM
 void RarVM::DecodeArg(VM_PreparedOperand &Op,bool ByteMode)
 {
   uint Data=fgetbits();
   if (Data & 0x8000)
   {
-    Op.Type=VM_OPREG;
-    Op.Data=(Data>>12)&7;
-    Op.Addr=&R[Op.Data];
-    faddbits(4);
+    Op.Type=VM_OPREG;     // Operand is register (R[0]..R[7])
+    Op.Data=(Data>>12)&7; // Register number
+    Op.Addr=&R[Op.Data];  // Register address
+    faddbits(4);          // 1 flag bit and 3 register number bits 
   }
   else
     if ((Data & 0xc000)==0)
     {
-      Op.Type=VM_OPINT;
+      Op.Type=VM_OPINT;   // Operand is integer
       if (ByteMode)
       {
-        Op.Data=(Data>>6) & 0xff;
+        Op.Data=(Data>>6) & 0xff;  // Byte integer.
         faddbits(10);
       }
       else
       {
         faddbits(2);
-        Op.Data=ReadData(*this);
+        Op.Data=ReadData(*this);   // 32 bit integer.
       }
     }
     else
     {
+      // Operand is data addressed by register data, base address or both.
       Op.Type=VM_OPREGMEM;
       if ((Data & 0x2000)==0)
       {
+        // Base address is zero, just use the address from register.
         Op.Data=(Data>>10)&7;
         Op.Addr=&R[Op.Data];
         Op.Base=0;
@@ -624,19 +717,22 @@ void RarVM::DecodeArg(VM_PreparedOperand &Op,bool ByteMode)
       {
         if ((Data & 0x1000)==0)
         {
+          // Use both register and base address.
           Op.Data=(Data>>9)&7;
           Op.Addr=&R[Op.Data];
           faddbits(7);
         }
         else
         {
+          // Use base address only. Access memory by fixed address.
           Op.Data=0;
           faddbits(4);
         }
-        Op.Base=ReadData(*this);
+        Op.Base=ReadData(*this); // Read base address.
       }
     }
 }
+#endif
 
 
 uint RarVM::ReadData(BitInput &Inp)
@@ -675,7 +771,7 @@ uint RarVM::ReadData(BitInput &Inp)
 }
 
 
-void RarVM::SetMemory(unsigned int Pos,byte *Data,unsigned int DataSize)
+void RarVM::SetMemory(size_t Pos,byte *Data,size_t DataSize)
 {
   if (Pos<VM_MEMSIZE && Data!=Mem+Pos)
     memmove(Mem+Pos,Data,Min(DataSize,VM_MEMSIZE-Pos));
@@ -686,11 +782,14 @@ void RarVM::SetMemory(unsigned int Pos,byte *Data,unsigned int DataSize)
 void RarVM::Optimize(VM_PreparedProgram *Prg)
 {
   VM_PreparedCommand *Code=&Prg->Cmd[0];
-  int CodeSize=Prg->CmdCount;
+  uint CodeSize=Prg->CmdCount;
 
-  for (int I=0;I<CodeSize;I++)
+  for (uint I=0;I<CodeSize;I++)
   {
     VM_PreparedCommand *Cmd=Code+I;
+
+    // Replace universal opcodes with faster byte or word only processing
+    // opcodes.
     switch(Cmd->OpCode)
     {
       case VM_MOV:
@@ -702,8 +801,13 @@ void RarVM::Optimize(VM_PreparedProgram *Prg)
     }
     if ((VM_CmdFlags[Cmd->OpCode] & VMCF_CHFLAGS)==0)
       continue;
+
+    // If we do not have jump commands between the current operation
+    // and next command which will modify processor flags, we can replace
+    // the current command with faster version which does not need to
+    // modify flags.
     bool FlagsRequired=false;
-    for (int J=I+1;J<CodeSize;J++)
+    for (uint J=I+1;J<CodeSize;J++)
     {
       int Flags=VM_CmdFlags[Code[J].OpCode];
       if (Flags & (VMCF_JUMP|VMCF_PROC|VMCF_USEFLAGS))
@@ -714,6 +818,10 @@ void RarVM::Optimize(VM_PreparedProgram *Prg)
       if (Flags & VMCF_CHFLAGS)
         break;
     }
+
+    // Below we'll replace universal opcodes with faster byte or word only 
+    // processing opcodes, which also do not modify processor flags to
+    // provide better performance.
     if (FlagsRequired)
       continue;
     switch(Cmd->OpCode)
@@ -740,24 +848,23 @@ void RarVM::Optimize(VM_PreparedProgram *Prg)
 
 
 #ifdef VM_STANDARDFILTERS
-VM_StandardFilters RarVM::IsStandardFilter(byte *Code,int CodeSize)
+VM_StandardFilters RarVM::IsStandardFilter(byte *Code,uint CodeSize)
 {
   struct StandardFilterSignature
   {
     int Length;
     uint CRC;
     VM_StandardFilters Type;
-  } StdList[]={
+  } static StdList[]={
     53, 0xad576887, VMSF_E8,
     57, 0x3cd7e57e, VMSF_E8E9,
    120, 0x3769893f, VMSF_ITANIUM,
     29, 0x0e06077d, VMSF_DELTA,
    149, 0x1c2c5dc8, VMSF_RGB,
-   216, 0xbc85e701, VMSF_AUDIO,
-    40, 0x46b9c560, VMSF_UPCASE
+   216, 0xbc85e701, VMSF_AUDIO
   };
-  uint CodeCRC=CRC(0xffffffff,Code,CodeSize)^0xffffffff;
-  for (int I=0;I<sizeof(StdList)/sizeof(StdList[0]);I++)
+  uint CodeCRC=CRC32(0xffffffff,Code,CodeSize)^0xffffffff;
+  for (uint I=0;I<ASIZE(StdList);I++)
     if (StdList[I].CRC==CodeCRC && StdList[I].Length==CodeSize)
       return(StdList[I].Type);
   return(VMSF_NONE);
@@ -775,20 +882,20 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         int DataSize=R[4];
         uint FileOffset=R[6];
 
-        if (DataSize>=VM_GLOBALMEMADDR)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR || DataSize<4)
           break;
 
         const int FileSize=0x1000000;
         byte CmpByte2=FilterType==VMSF_E8E9 ? 0xe9:0xe8;
-        for (uint CurPos=0;CurPos<DataSize-4;)
+        for (int CurPos=0;CurPos<DataSize-4;)
         {
           byte CurByte=*(Data++);
           CurPos++;
           if (CurByte==0xe8 || CurByte==CmpByte2)
           {
 #ifdef PRESENT_INT32
-            sint32 Offset=CurPos+FileOffset;
-            sint32 Addr=GET_VALUE(false,Data);
+            int32 Offset=CurPos+FileOffset;
+            int32 Addr=GET_VALUE(false,Data);
             if (Addr<0)
             {
               if (Addr+Offset>=0)
@@ -821,10 +928,10 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         int DataSize=R[4];
         uint FileOffset=R[6];
 
-        if (DataSize>=VM_GLOBALMEMADDR)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR || DataSize<21)
           break;
 
-        uint CurPos=0;
+        int CurPos=0;
 
         FileOffset>>=4;
 
@@ -858,8 +965,11 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
       {
         int DataSize=R[4],Channels=R[0],SrcPos=0,Border=DataSize*2;
         SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-        if (DataSize>=VM_GLOBALMEMADDR/2)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR/2)
           break;
+
+        // Bytes from same channels are grouped to continual data blocks,
+        // so we need to place them back to their interleaving positions.
         for (int CurChannel=0;CurChannel<Channels;CurChannel++)
         {
           byte PrevByte=0;
@@ -874,21 +984,21 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         byte *SrcData=Mem,*DestData=SrcData+DataSize;
         const int Channels=3;
         SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-        if (DataSize>=VM_GLOBALMEMADDR/2)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR/2 || PosR<0)
           break;
         for (int CurChannel=0;CurChannel<Channels;CurChannel++)
         {
-          unsigned int PrevByte=0;
+          uint PrevByte=0;
 
           for (int I=CurChannel;I<DataSize;I+=Channels)
           {
-            unsigned int Predicted;
+            uint Predicted;
             int UpperPos=I-Width;
             if (UpperPos>=3)
             {
               byte *UpperData=DestData+UpperPos;
-              unsigned int UpperByte=*UpperData;
-              unsigned int UpperLeftByte=*(UpperData-3);
+              uint UpperByte=*UpperData;
+              uint UpperLeftByte=*(UpperData-3);
               Predicted=PrevByte+UpperByte-UpperLeftByte;
               int pa=abs((int)(Predicted-PrevByte));
               int pb=abs((int)(Predicted-UpperByte));
@@ -919,11 +1029,11 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         int DataSize=R[4],Channels=R[0];
         byte *SrcData=Mem,*DestData=SrcData+DataSize;
         SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-        if (DataSize>=VM_GLOBALMEMADDR/2)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR/2)
           break;
         for (int CurChannel=0;CurChannel<Channels;CurChannel++)
         {
-          unsigned int PrevByte=0,PrevDelta=0,Dif[7];
+          uint PrevByte=0,PrevDelta=0,Dif[7];
           int D1=0,D2=0,D3;
           int K1=0,K2=0,K3=0;
           memset(Dif,0,sizeof(Dif));
@@ -934,10 +1044,10 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
             D2=PrevDelta-D1;
             D1=PrevDelta;
 
-            unsigned int Predicted=8*PrevByte+K1*D1+K2*D2+K3*D3;
+            uint Predicted=8*PrevByte+K1*D1+K2*D2+K3*D3;
             Predicted=(Predicted>>3) & 0xff;
 
-            unsigned int CurByte=*(SrcData++);
+            uint CurByte=*(SrcData++);
 
             Predicted-=CurByte;
             DestData[I]=Predicted;
@@ -956,7 +1066,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
 
             if ((ByteCount & 0x1f)==0)
             {
-              unsigned int MinDif=Dif[0],NumMinDif=0;
+              uint MinDif=Dif[0],NumMinDif=0;
               Dif[0]=0;
               for (int J=1;J<sizeof(Dif)/sizeof(Dif[0]);J++)
               {
@@ -981,31 +1091,15 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         }
       }
       break;
-    case VMSF_UPCASE:
-      {
-        int DataSize=R[4],SrcPos=0,DestPos=DataSize;
-        if (DataSize>=VM_GLOBALMEMADDR/2)
-          break;
-        while (SrcPos<DataSize)
-        {
-          byte CurByte=Mem[SrcPos++];
-          if (CurByte==2 && (CurByte=Mem[SrcPos++])!=2)
-            CurByte-=32;
-          Mem[DestPos++]=CurByte;
-        }
-        SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x1c],DestPos-DataSize);
-        SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-      }
-      break;
   }
 }
 
 
-unsigned int RarVM::FilterItanium_GetBits(byte *Data,int BitPos,int BitCount)
+uint RarVM::FilterItanium_GetBits(byte *Data,int BitPos,int BitCount)
 {
   int InAddr=BitPos/8;
   int InBit=BitPos&7;
-  unsigned int BitField=(uint)Data[InAddr++];
+  uint BitField=(uint)Data[InAddr++];
   BitField|=(uint)Data[InAddr++] << 8;
   BitField|=(uint)Data[InAddr++] << 16;
   BitField|=(uint)Data[InAddr] << 24;
@@ -1014,17 +1108,16 @@ unsigned int RarVM::FilterItanium_GetBits(byte *Data,int BitPos,int BitCount)
 }
 
 
-void RarVM::FilterItanium_SetBits(byte *Data,unsigned int BitField,int BitPos,
-                                  int BitCount)
+void RarVM::FilterItanium_SetBits(byte *Data,uint BitField,int BitPos,int BitCount)
 {
   int InAddr=BitPos/8;
   int InBit=BitPos&7;
-  unsigned int AndMask=0xffffffff>>(32-BitCount);
+  uint AndMask=0xffffffff>>(32-BitCount);
   AndMask=~(AndMask<<InBit);
 
   BitField<<=InBit;
 
-  for (int I=0;I<4;I++)
+  for (uint I=0;I<4;I++)
   {
     Data[InAddr+I]&=AndMask;
     Data[InAddr+I]|=BitField;
