@@ -44,18 +44,31 @@ A million repetitions of "a"
 #define R3(v,w,x,y,z,i) {z+=(((w|x)&y)|(w&x))+blk(i)+0x8F1BBCDC+rol(v,5);w=rol(w,30);}
 #define R4(v,w,x,y,z,i) {z+=(w^x^y)+blk(i)+0xCA62C1D6+rol(v,5);w=rol(w,30);}
 
+#ifdef _MSC_VER
+#pragma optimize( "", off )
+// We need to disable the optimization to really wipe these variables.
+#endif
+static void wipevars(uint32 &a,uint32 &b,uint32 &c,uint32 &d,uint32 &e)
+{
+  // Wipe used variables for safety reason.
+  a=b=c=d=e=0;
+}
+#ifdef _MSC_VER
+#pragma optimize( "", on )
+#endif
 
 /* Hash a single 512-bit block. This is the core of the algorithm. */
 
-void SHA1Transform(uint32 state[5], unsigned char buffer[64], bool handsoff)
+void SHA1Transform(uint32 state[5], unsigned char workspace[64], const byte buffer[64], bool handsoff)
 {
+#ifndef SFX_MODULE
   uint32 a, b, c, d, e;
+#endif
   typedef union {
     unsigned char c[64];
     uint32 l[16];
 } CHAR64LONG16;
 CHAR64LONG16* block;
-static unsigned char workspace[64];
     if (handsoff)
     {
       block = (CHAR64LONG16*)workspace;
@@ -131,15 +144,18 @@ static unsigned char workspace[64];
     state[4] += e;
 
     /* Wipe variables */
-    a = b = c = d = e = 0;
-    memset(&a,0,sizeof(a));
+// Such wipe method does not work in optimizing compilers.
+//    a = b = c = d = e = 0;
+//    memset(&a,0,sizeof(a));
+
+    wipevars(a,b,c,d,e);
 #endif
 }
 
 
 /* Initialize new context */
 
-void hash_initial(hash_context* context)
+void sha1_init(sha1_context* context)
 {
     /* SHA1 initialization constants */
     context->state[0] = 0x67452301;
@@ -152,7 +168,7 @@ void hash_initial(hash_context* context)
 
 
 /* Run your data through this. */
-void hash_process( hash_context * context, unsigned char * data, unsigned len,
+void sha1_process( sha1_context * context, const unsigned char * data, size_t len,
                    bool handsoff )
 {
 unsigned int i, j;
@@ -160,18 +176,19 @@ uint blen = ((uint)len)<<3;
 
     j = (context->count[0] >> 3) & 63;
     if ((context->count[0] += blen) < blen ) context->count[1]++;
-    context->count[1] += (len >> 29);
+    context->count[1] += (uint32)(len >> 29);
     if ((j + len) > 63) {
         memcpy(&context->buffer[j], data, (i = 64-j));
-        SHA1Transform(context->state, context->buffer, handsoff);
+        SHA1Transform(context->state, context->workspace, context->buffer, handsoff);
         for ( ; i + 63 < len; i += 64) {
-#ifdef ALLOW_NOT_ALIGNED_INT
-            SHA1Transform(context->state, &data[i], handsoff);
+#ifdef ALLOW_MISALIGNED
+            SHA1Transform(context->state, context->workspace, &data[i], handsoff);
 #else
-            unsigned char buffer[64];
+            uint buffer[16];
             memcpy(buffer,data+i,sizeof(buffer));
-            SHA1Transform(context->state, buffer, handsoff);
-            memcpy(data+i,buffer,sizeof(buffer));
+            SHA1Transform(context->state, context->workspace, (byte*)buffer, handsoff);
+            if (!handsoff)
+              memcpy((byte *)(data+i),buffer,sizeof(buffer));
 #endif
 #ifdef BIG_ENDIAN
             if (!handsoff)
@@ -198,7 +215,7 @@ uint blen = ((uint)len)<<3;
 
 /* Add padding and return the message digest. */
 
-void hash_final( hash_context* context, uint32 digest[5], bool handsoff)
+void sha1_done( sha1_context* context, uint32 digest[5], bool handsoff)
 {
 uint i, j;
 unsigned char finalcount[8];
@@ -207,25 +224,26 @@ unsigned char finalcount[8];
         finalcount[i] = (unsigned char)((context->count[(i >= 4 ? 0 : 1)]
          >> ((3-(i & 3)) * 8) ) & 255);  /* Endian independent */
     }
-    unsigned char ch='\200';
-    hash_process(context, &ch, 1, handsoff);
+    unsigned char ch=(unsigned char)'\200';
+    sha1_process(context, &ch, 1, handsoff);
     while ((context->count[0] & 504) != 448) {
         ch=0;
-        hash_process(context, &ch, 1, handsoff);
+        sha1_process(context, &ch, 1, handsoff);
     }
-    hash_process(context, finalcount, 8, handsoff);  /* Should cause a SHA1Transform() */
+    sha1_process(context, finalcount, 8, handsoff);  /* Should cause a SHA1Transform() */
     for (i = 0; i < 5; i++) {
         digest[i] = context->state[i] & 0xffffffff;
     }
     /* Wipe variables */
-    memset(&i,0,sizeof(i));
-    memset(&j,0,sizeof(j));
-    memset(context->buffer, 0, 64);
-    memset(context->state, 0, 20);
-    memset(context->count, 0, 8);
-    memset(&finalcount, 0, 8);
+    cleandata(&i,sizeof(i));
+    cleandata(&j,sizeof(j));
+    cleandata(context->buffer, 64);
+    cleandata(context->state, 20);
+    cleandata(context->count, 8);
+    cleandata(&finalcount, 8);
     if (handsoff)
-      SHA1Transform(context->state, context->buffer, true);
+      memset(context->workspace,0,sizeof(context->workspace)); // Wipe the temporary buffer.
+//      SHA1Transform(context->state, context->workspace, context->buffer, true); /* make SHA1Transform overwrite it's own static vars */
 }
 
 
