@@ -2,13 +2,13 @@
 
 const char *NullToEmpty(const char *Str)
 {
-  return(Str==NULL ? "":Str);
+  return Str==NULL ? "":Str;
 }
 
 
 const wchar *NullToEmpty(const wchar *Str)
 {
-  return(Str==NULL ? L"":Str);
+  return Str==NULL ? L"":Str;
 }
 
 
@@ -17,6 +17,10 @@ void IntToExt(const char *Src,char *Dest,size_t DestSize)
 #ifdef _WIN_ALL
   OemToCharBuffA(Src,Dest,(DWORD)DestSize);
   Dest[DestSize-1]=0;
+#elif defined(_ANDROID)
+  wchar DestW[NM];
+  UnkToWide(Src,DestW,ASIZE(DestW));
+  WideToChar(DestW,Dest,DestSize);
 #else
   if (Dest!=Src)
     strncpyz(Dest,Src,DestSize);
@@ -27,7 +31,7 @@ void IntToExt(const char *Src,char *Dest,size_t DestSize)
 int stricomp(const char *s1,const char *s2)
 {
 #ifdef _WIN_ALL
-  return(CompareStringA(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,-1,s2,-1)-2);
+  return CompareStringA(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,-1,s2,-1)-2;
 #else
   while (toupper(*s1)==toupper(*s2))
   {
@@ -194,7 +198,7 @@ uint GetDigits(uint Number)
     Number/=10;
     Digits++;
   }
-  return(Digits);
+  return Digits;
 }
 #endif
 
@@ -203,8 +207,8 @@ bool LowAscii(const char *Str)
 {
   for (int I=0;Str[I]!=0;I++)
     if ((byte)Str[I]<32 || (byte)Str[I]>127)
-      return(false);
-  return(true);
+      return false;
+  return true;
 }
 
 
@@ -213,20 +217,20 @@ bool LowAscii(const wchar *Str)
   for (int I=0;Str[I]!=0;I++)
   {
     // We convert wchar_t to uint just in case if some compiler
-    // uses the signed wchar_t.
+    // uses signed wchar_t.
     if ((uint)Str[I]<32 || (uint)Str[I]>127)
-      return(false);
+      return false;
   }
-  return(true);
+  return true;
 }
 
 
 int wcsicompc(const wchar *Str1,const wchar *Str2)
 {
 #if defined(_UNIX)
-  return(wcscmp(Str1,Str2));
+  return wcscmp(Str1,Str2);
 #else
-  return(wcsicomp(Str1,Str2));
+  return wcsicomp(Str1,Str2);
 #endif
 }
 
@@ -239,7 +243,7 @@ char* strncpyz(char *dest, const char *src, size_t maxlen)
     strncpy(dest,src,maxlen-1);
     dest[maxlen-1]=0;
   }
-  return(dest);
+  return dest;
 }
 
 
@@ -323,7 +327,7 @@ const wchar* GetWide(const char *Src)
   wchar *Str=StrTable[StrNum];
   CharToWide(Src,Str,MaxLength);
   Str[MaxLength-1]=0;
-  return(Str);
+  return Str;
 }
 
 
@@ -398,14 +402,3 @@ void PrintfPrepareFmt(const wchar *Org,wchar *Cvt,size_t MaxSize)
   Cvt[Dest]=0;
 }
 #endif
-
-
-bool GetPassword(PASSWORD_TYPE Type,const wchar *FileName,SecPassword *Password)
-{
-#ifdef SILENT
-  return false;
-#else
-  return GetConsolePassword(Type,FileName,Password);
-#endif
-}
-
