@@ -1,52 +1,29 @@
 #ifndef _RAR_UNICODE_
 #define _RAR_UNICODE_
 
-#ifndef _EMX
-#define MBFUNCTIONS
-#endif
-
-#if defined(MBFUNCTIONS) || defined(_WIN_ALL) || defined(_EMX) && !defined(_DJGPP)
-#define UNICODE_SUPPORTED
-#endif
-
-#if !defined(SFX_MODULE) && (defined(_MSC_VER) || defined(__BORLANDC__))
-// If C_UNICODE_RTL is defined, we can use library Unicode functions like
-// wcscpy. Otherwise, for compatibility with old compilers or for removing
-// RTL to reduce SFX module size, we need need to use our own implementations.
-#define C_UNICODE_RTL
-#endif
-
 #ifdef _WIN_ALL
 #define DBCS_SUPPORTED
 #endif
 
-#ifdef _EMX
-int uni_init(int codepage);
-int uni_done();
-#endif
-
-#ifdef __BORLANDC__
-// Borland C++ Builder 5 uses the old style swprintf without the buffer size,
-// so we replace it with snwprintf in our custom sprintfw definition.
-#define sprintfw snwprintf
-#else
-#define sprintfw swprintf
-#endif
-
-bool WideToChar(const wchar *Src,char *Dest,size_t DestSize=0x1000000);
-bool CharToWide(const char *Src,wchar *Dest,size_t DestSize=0x1000000);
-byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize=0x1000000);
-wchar* RawToWide(const byte *Src,wchar *Dest,size_t DestSize=0x1000000);
+bool WideToChar(const wchar *Src,char *Dest,size_t DestSize);
+bool CharToWide(const char *Src,wchar *Dest,size_t DestSize);
+byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize);
+wchar* RawToWide(const byte *Src,wchar *Dest,size_t DestSize);
 void WideToUtf(const wchar *Src,char *Dest,size_t DestSize);
-void UtfToWide(const char *Src,wchar *Dest,size_t DestSize);
-bool UnicodeEnabled();
+size_t WideToUtfSize(const wchar *Src);
+bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize);
 
 int wcsicomp(const wchar *s1,const wchar *s2);
 int wcsnicomp(const wchar *s1,const wchar *s2,size_t n);
-wchar* wcslower(wchar *Str);
-wchar* wcsupper(wchar *Str);
+const wchar_t* wcscasestr(const wchar_t *str, const wchar_t *search);
+#ifndef SFX_MODULE
+wchar* wcslower(wchar *s);
+wchar* wcsupper(wchar *s);
+#endif
 int toupperw(int ch);
-int atoiw(const wchar *s);
+int tolowerw(int ch);
+uint atoiw(const wchar *s);
+uint64 atoilw(const wchar *s);
 
 #ifdef DBCS_SUPPORTED
 class SupportDBCS
