@@ -52,7 +52,10 @@ void Unpack::Unpack5(bool Solid)
     uint MainSlot=DecodeNumber(Inp,&BlockTables.LD);
     if (MainSlot<256)
     {
-      Window[UnpPtr++]=(byte)MainSlot;
+      if (Fragmented)
+        FragWindow[UnpPtr++]=(byte)MainSlot;
+      else
+        Window[UnpPtr++]=(byte)MainSlot;
       continue;
     }
     if (MainSlot>=262)
@@ -103,7 +106,10 @@ void Unpack::Unpack5(bool Solid)
 
       InsertOldDist(Distance);
       LastLength=Length;
-      CopyString(Length,Distance);
+      if (Fragmented)
+        FragWindow.CopyString(Length,Distance,UnpPtr,MaxWinMask);
+      else
+        CopyString(Length,Distance);
       continue;
     }
     if (MainSlot==256)
@@ -116,7 +122,10 @@ void Unpack::Unpack5(bool Solid)
     if (MainSlot==257)
     {
       if (LastLength!=0)
-        CopyString(LastLength,OldDist[0]);
+        if (Fragmented)
+          FragWindow.CopyString(LastLength,OldDist[0],UnpPtr,MaxWinMask);
+        else
+          CopyString(LastLength,OldDist[0]);
       continue;
     }
     if (MainSlot<262)
@@ -130,7 +139,10 @@ void Unpack::Unpack5(bool Solid)
       uint LengthSlot=DecodeNumber(Inp,&BlockTables.RD);
       uint Length=SlotToLength(Inp,LengthSlot);
       LastLength=Length;
-      CopyString(Length,Distance);
+      if (Fragmented)
+        FragWindow.CopyString(Length,Distance,UnpPtr,MaxWinMask);
+      else
+        CopyString(Length,Distance);
       continue;
     }
   }
@@ -221,13 +233,19 @@ bool Unpack::UnpReadBuf()
   }
   else
     DataSize=ReadTop;
-  int ReadCode=UnpIO->UnpRead(Inp.InBuf+DataSize,BitInput::MAX_SIZE-DataSize);
-  if (ReadCode>0)
+  int ReadCode=0;
+  if (BitInput::MAX_SIZE!=DataSize)
+    ReadCode=UnpIO->UnpRead(Inp.InBuf+DataSize,BitInput::MAX_SIZE-DataSize);
+  if (ReadCode>0) // Can be also -1.
     ReadTop+=ReadCode;
   ReadBorder=ReadTop-30;
   BlockHeader.BlockStart=Inp.InAddr;
   if (BlockHeader.BlockSize!=-1) // '-1' means not defined yet.
+  {
+    // We may need to quit from main extraction loop and read new block header
+    // and trees earlier than data in input buffer ends.
     ReadBorder=Min(ReadBorder,BlockHeader.BlockStart+BlockHeader.BlockSize-1);
+  }
   return ReadCode!=-1;
 }
 
@@ -286,12 +304,25 @@ void Unpack::UnpWriteBuf()
           FilterSrcMemory.Alloc(BlockLength);
           byte *Mem=&FilterSrcMemory[0];
           if (BlockStart<BlockEnd || BlockEnd==0)
-            memcpy(Mem,Window+BlockStart,BlockLength);
+          {
+            if (Fragmented)
+              FragWindow.CopyData(Mem,BlockStart,BlockLength);
+            else
+              memcpy(Mem,Window+BlockStart,BlockLength);
+          }
           else
           {
             size_t FirstPartLength=size_t(MaxWinSize-BlockStart);
-            memcpy(Mem,Window+BlockStart,FirstPartLength);
-            memcpy(Mem+FirstPartLength,Window,BlockEnd);
+            if (Fragmented)
+            {
+              FragWindow.CopyData(Mem,BlockStart,FirstPartLength);
+              FragWindow.CopyData(Mem+FirstPartLength,0,BlockEnd);
+            }
+            else
+            {
+              memcpy(Mem,Window+BlockStart,FirstPartLength);
+              memcpy(Mem+FirstPartLength,Window,BlockEnd);
+            }
           }
 
           byte *OutMem=ApplyFilter(Mem,BlockLength,flt);
@@ -395,7 +426,7 @@ void Unpack::FilterItanium_SetBits(byte *Data,uint BitField,int BitPos,int BitCo
 
 inline uint GetFiltData32(byte *Data)
 {
-#if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT) || !defined(PRESENT_INT32)
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED) || !defined(PRESENT_INT32)
   uint Value=GET_UINT32((uint)Data[0]|((uint)Data[1]<<8)|((uint)Data[2]<<16)|((uint)Data[3]<<24));
 #else
   uint Value=GET_UINT32(*(uint32 *)Data);
@@ -406,7 +437,7 @@ inline uint GetFiltData32(byte *Data)
 
 inline void SetFiltData32(byte *Data,uint Value)
 {
-#if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT) || !defined(PRESENT_INT32)
+#if defined(BIG_ENDIAN) || !defined(ALLOW_MISALIGNED) || !defined(PRESENT_INT32)
    Data[0]=(byte)Value;
    Data[1]=(byte)(Value>>8);
    Data[2]=(byte)(Value>>16);
@@ -652,13 +683,27 @@ void Unpack::UnpWriteArea(size_t StartPtr,size_t EndPtr)
   if (EndPtr!=StartPtr)
     UnpSomeRead=true;
   if (EndPtr<StartPtr)
-  {
-    UnpWriteData(&Window[StartPtr],-(int)StartPtr & MaxWinMask);
-    UnpWriteData(Window,EndPtr);
     UnpAllBuf=true;
+
+  if (Fragmented)
+  {
+    size_t SizeToWrite=(EndPtr-StartPtr) & MaxWinMask;
+    while (SizeToWrite>0)
+    {
+      size_t BlockSize=FragWindow.GetBlockSize(StartPtr,SizeToWrite);
+      UnpWriteData(&FragWindow[StartPtr],BlockSize);
+      SizeToWrite-=BlockSize;
+      StartPtr=(StartPtr+BlockSize) & MaxWinMask;
+    }
   }
   else
-    UnpWriteData(&Window[StartPtr],EndPtr-StartPtr);
+    if (EndPtr<StartPtr)
+    {
+      UnpWriteData(Window+StartPtr,MaxWinSize-StartPtr);
+      UnpWriteData(Window,EndPtr);
+    }
+    else
+      UnpWriteData(Window+StartPtr,EndPtr-StartPtr);
 }
 
 
